@@ -34,11 +34,11 @@
 #'   \item{\code{data}: }{Required if variables are not in \code{parent.frame()} or 
 #'       obtainable via \code{object$data}}
 #'   \item{\code{coef}: }{\code{coef(object)}}
-#'   \item{\code{mcmc}: }{\code{object$sample}}
 #'   \item{\code{vcov}: }{\code{vcov(object)}}
 #'   \item{\code{df}: }{Set to \code{Inf} if not available in \code{object$df.residual}}
+#'   \item{\code{mcmc}: }{\code{object$sample}}
 #'   \item{\code{subset}: }{\code{NULL} (so that all observations in \code{data} are used)}
-#'   \item{\code{contrasts}: }{\code{NULL} (so that \code{getOption("contrasts")} is used)}
+#'   \item{\code{contrasts}: }{\code{object$contrasts}}
 #' }
 #' 
 #' The functions \code{\link{qdrg}} and \code{emmobj} are close cousins, in that
@@ -49,11 +49,11 @@
 #' @param formula Formula for the fixed effects
 #' @param data Dataset containing the variables in the model
 #' @param coef Fixed-effect regression coefficients (must conform to formula)
-#' @param mcmc Posterior sample of fixed-effect coefficients
 #' @param vcov Variance-covariance matrix of the fixed effects
+#' @param df Error degrees of freedom
+#' @param mcmc Posterior sample of fixed-effect coefficients
 #' @param object Optional model object. If provided, it is used to set 
 #'     certain other arguments, if not specified. See Details.
-#' @param df Error degrees of freedom
 #' @param subset Subset of \code{data} used in fitting the model
 #' @param weights Weights used in fitting the model
 #' @param contrasts List of contrasts specified in fitting the model
@@ -61,6 +61,12 @@
 #'     (Note: response transformations are auto-detected from \code{formula})
 #' @param qr QR decomposition of the model matrix; needed only if there are \code{NA}s
 #'     in \code{coef}.
+#' @param ordinal.dim Integer number of levels in an ordinal response. If not
+#'     missing, the intercept terms are modified appropriate to predicting the latent
+#'     response (see \code{vignette("models")}, Section O. In this case, we expect
+#'     the first \code{ordinal.dim - 1} elements of \code{coef} to be the
+#'     estimated threshold parameters, followed by the coefficients for the
+#'     linear predictor.)
 #' @param ... Optional arguments passed to \code{\link{ref_grid}}
 #'
 #' @return An \code{emmGrid} object constructed from the arguments
@@ -72,21 +78,31 @@
 #' if (require(biglm)) {
 #'   # Post hoc analysis of a "biglm" object -- not supported by emmeans
 #'   bigmod <- biglm(log(conc) ~ source + factor(percent), data = pigs)
-#'    
-#'   rg2 <- qdrg(object = bigmod, data = pigs)
-#'   summary(emmeans(rg2, "source"), type = "response")
+#'   
+#'   rg1 <- qdrg(log(conc) ~ source + factor(percent), data = pigs, 
+#'       coef = coef(bigmod), vcov = vcov(bigmod), df = bigmod$df.residual)
+#'   emmeans(rg1, "source", type = "response")
+#'   
+#'   ## But in this particular case, we could have done it the easy way:
+#'   ##     rg1 <- qdrg(object = bigmod, data = pigs)
 #' }
 #' if(require(coda) && require(lme4)) {
 #'   # Use a stored example having a posterior sample
 #'   # Model is based on the data in lme4::cbpp
 #'   
 #'   post <- readRDS(system.file("extdata", "cbpplist", package = "emmeans"))$post.beta
-#'   rg1 <- qdrg(~ size + period, data = lme4::cbpp, mcmc = post, link = "logit")
-#'   summary(rg1, type = "response")
+#'   rg2 <- qdrg(~ size + period, data = lme4::cbpp, mcmc = post, link = "logit")
+#'   summary(rg2, type = "response")
+#' }
+#' if(require(ordinal)) {
+#'   wine.clm <- clm(rating ~ temp * contact, data = wine)
+#'   ref_grid(wine.clm)
+#'   # verify that we get the same thing via:
+#'   qdrg(object = wine.clm, data = wine, ordinal.dim = 5)
 #' }
 #'
-qdrg = function(formula, data, coef, mcmc, vcov, object,
-                df, subset, weights, contrasts, link, qr, ...) {
+qdrg = function(formula, data, coef, vcov, df, mcmc, object,
+                subset, weights, contrasts, link, qr, ordinal.dim, ...) {
     result = match.call(expand.dots = FALSE)
     if (!missing(object)) {
         if (missing(formula)) 
@@ -104,6 +120,8 @@ qdrg = function(formula, data, coef, mcmc, vcov, object,
             result$vcov = stats::vcov(object)
         if(missing(df))
             result$df = object$df.residual
+        if(missing(contrasts))
+            result$contrasts = object$contrasts
         if(any(is.na(result$coef)))
             result$qr = object$qr
     }
@@ -127,6 +145,7 @@ qdrg = function(formula, data, coef, mcmc, vcov, object,
     if(!missing(contrasts)) result$contrasts = contrasts
     if(!missing(link)) result$link = link
     if(!missing(qr) && any(is.na(result$coef))) result$qr = qr
+    if(!missing(ordinal.dim)) result$ordinal.dim = ordinal.dim
     
     # make sure "formula" exists, has a LHS and is is 2nd element so that 
     # response transformation can be found
@@ -152,13 +171,18 @@ emm_basis.qdrg = function(object, trms, xlev, grid, ...) {
     bhat = object$coef
     m = suppressWarnings(model.frame(trms, grid, na.action = na.pass, xlev = xlev))
     X = model.matrix(trms, m, contrasts.arg = object$contrasts)
-    bhat = as.numeric(bhat) 
     V = .my.vcov(object, ...)
     if (!is.null(object$mcmc)) {
         if (is.null(object$coef)) bhat = apply(object$mcmc, 2, mean)
         if (is.null(object$vcov)) V = cov(object$mcmc)
     }
     
+    # If ordinal, add extra avgd, subtracted intercepts -- for latent mode
+    if(!is.null(od <- object$ordinal.dim)) { 
+        intcpt = matrix(-1 / (od - 1), nrow = nrow(X), ncol = od - 1)
+        X = cbind(intcpt, X[, -1, drop = FALSE])
+    }
+    
     nbasis = estimability::all.estble
     if (sum(is.na(bhat)) > 0) {
         if(!is.na(object$qr))
@@ -169,6 +193,18 @@ emm_basis.qdrg = function(object, trms, xlev, grid, ...) {
     }
     
     misc = list()
+    
+    # check multivariate situation
+    if (is.matrix(bhat)) {
+        X = kronecker (diag(ncol(bhat)), X)
+        nbasis = kronecker(rep(1, ncol(bhat)), nbasis)
+        nms = colnames(bhat)
+        if (is.null(nms))
+            nms = seq_len(ncol(bhat))
+        misc$ylevs = list(rep.meas = nms)
+        bhat = as.numeric(bhat)
+    }
+    
     if (!is.null(object$link)) {
         misc = .std.link.labels(eval(list(link = object$link)), misc)
         dffun = function(k, dfargs) Inf
